@@ -1,15 +1,15 @@
 /*
  * Copyright © 2017, 2018 Christian Persch
  *
- * This programme is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * This programme is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
@@ -22,6 +22,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#include <bitset>
 #include <cassert>
 #include <cstring>
 #include <cerrno>
@@ -31,9 +32,13 @@
 #include <string>
 
 #include "debug.h"
+#include "glib-glue.hh"
+#include "libc-glue.hh"
 #include "parser.hh"
 #include "parser-glue.hh"
+#include "std-glue.hh"
 #include "utf8.hh"
+#include "vtedefines.hh"
 
 enum {
 #define _VTE_SGR(...)
@@ -45,6 +50,12 @@ enum {
 
 using namespace std::literals;
 
+enum class DataSyntax {
+        ECMA48_UTF8,
+        /* ECMA48_PCTERM, */
+        /* ECMA48_ECMA35, */
+};
+
 char*
 vte::parser::Sequence::ucs4_to_utf8(gunichar const* str,
                                     ssize_t len) const noexcept
@@ -175,6 +186,13 @@ private:
                 { }
         };
 
+        class GreenAttr : private Attribute {
+        public:
+                GreenAttr(PrettyPrinter* printer)
+                        : Attribute(printer, "\e[7;32m"s, "\e[27;39m"s)
+                { }
+        };
+
         void
         print_params(vte::parser::Sequence const& seq) noexcept
         {
@@ -237,6 +255,12 @@ private:
                 m_str.append(buf, len);
         }
 
+        void
+        print_literal(char const* str) noexcept
+        {
+                m_str.append(str);
+        }
+
         G_GNUC_PRINTF(2, 3)
         void
         print_format(char const* format,
@@ -358,7 +382,11 @@ private:
         void
         printout() noexcept
         {
-                g_print("%s\n", m_str.c_str());
+                m_str.push_back('\n');
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-result"
+                write(STDOUT_FILENO, m_str.data(), m_str.size());
+#pragma GCC diagnostic pop
                 m_str.clear();
         }
 
@@ -376,13 +404,34 @@ public:
                 printout();
         }
 
-        void operator()(vte::parser::Sequence const& seq) noexcept
+        void VT(vte::parser::Sequence const& seq) noexcept
         {
                 print_seq(seq);
                 if (seq.command() == VTE_CMD_LF)
                         printout();
         }
 
+        void enter_data_syntax(DataSyntax syntax) noexcept
+        {
+                switch (syntax) {
+                default:
+                        break;
+                }
+        }
+
+        void leave_data_syntax(DataSyntax syntax,
+                               bool success) noexcept
+        {
+                switch (syntax) {
+                default:
+                        break;
+                }
+        }
+
+        void reset() noexcept
+        {
+        }
+
 }; // class PrettyPrinter
 
 class Linter {
@@ -396,7 +445,7 @@ private:
                 va_start(args, format);
                 char* str = g_strdup_vprintf(format, args);
                 va_end(args);
-                g_printerr("WARNING: %s\n", str);
+                g_print("WARNING: %s\n", str);
                 g_free(str);
         }
 
@@ -571,15 +620,15 @@ private:
         }
 
 public:
-        constexpr Linter() noexcept = default;
+        Linter() noexcept = default;
         ~Linter() noexcept = default;
 
-        void operator()(vte::parser::Sequence const& seq) noexcept
+        void VT(vte::parser::Sequence const& seq) noexcept
         {
                 auto cmd = seq.command();
                 switch (cmd) {
                 case VTE_CMD_OSC:
-                        if (seq.terminator() == 7 /* BEL */)
+                        if (seq.st() == 7 /* BEL */)
                                 warn("OSC terminated by BEL may be ignored; use ST (ESC \\) instead.");
                         break;
 
@@ -605,82 +654,184 @@ public:
                 }
         }
 
+        void enter_data_syntax(DataSyntax syntax) noexcept
+        {
+        }
+
+        void leave_data_syntax(DataSyntax syntax,
+                               bool success) noexcept
+        {
+        }
+
+        void reset() noexcept
+        {
+        }
+
 }; // class Linter
 
 class Sink {
 public:
-        void operator()(vte::parser::Sequence const& seq) noexcept { }
+        void VT(vte::parser::Sequence const& seq) noexcept { }
+
+        void enter_data_syntax(DataSyntax syntax) noexcept { }
+        void leave_data_syntax(DataSyntax syntax,
+                               bool success) noexcept { }
+
+        void reset() noexcept { }
 
 }; // class Sink
 
+template<class D>
 class Processor {
 private:
+        using Delegate = D;
+
+        D& m_delegate;
+        size_t m_buffer_size{0};
+        bool m_statistics{false};
+        bool m_benchmark{false};
+
         gsize m_seq_stats[VTE_SEQ_N];
         gsize m_cmd_stats[VTE_CMD_N];
         GArray* m_bench_times;
 
-        template<class Functor>
-        void
-        process_file_utf8(int fd,
-                          Functor& func)
-        {
-                vte::parser::Parser parser{};
-                vte::parser::Sequence seq{parser};
+        static constexpr const size_t k_buf_overlap = 1u;
 
-                gsize const buf_size = 16384;
-                guchar* buf = g_new0(guchar, buf_size);
+        vte::base::UTF8Decoder m_utf8_decoder{};
+        vte::parser::Parser m_parser{};
+
+        DataSyntax m_primary_data_syntax{DataSyntax::ECMA48_UTF8};
+        DataSyntax m_current_data_syntax{DataSyntax::ECMA48_UTF8};
+
+        void reset() noexcept
+        {
+                switch (m_current_data_syntax) {
+                case DataSyntax::ECMA48_UTF8:
+                        m_parser.reset();
+                        m_utf8_decoder.reset();
+                        break;
+
+                default:
+                        break;
+                }
+
+                if (m_current_data_syntax != m_primary_data_syntax) {
+                        m_current_data_syntax = m_primary_data_syntax;
+                        reset();
+                }
+
+                m_delegate.reset();
+        }
+
+        [[gnu::always_inline]]
+        bool
+        process_seq(vte::parser::Sequence const& seq) noexcept
+        {
+                m_delegate.VT(seq);
+                return true;
+        }
+
+        uint8_t const*
+        process_data_utf8(uint8_t const* const bufstart,
+                          uint8_t const* const bufend,
+                          bool eos) noexcept
+        {
+                auto seq = vte::parser::Sequence{m_parser};
+
+                for (auto sptr = bufstart; sptr < bufend; ) {
+                        switch (m_utf8_decoder.decode(*(sptr++))) {
+                        case vte::base::UTF8Decoder::REJECT_REWIND:
+                                /* Rewind the stream.
+                                 * Note that this will never lead to a loop, since in the
+                                 * next round this byte *will* be consumed.
+                                 */
+                                --sptr;
+                                [[fallthrough]];
+                        case vte::base::UTF8Decoder::REJECT:
+                                m_utf8_decoder.reset();
+                                /* Fall through to insert the U+FFFD replacement character. */
+                                [[fallthrough]];
+                        case vte::base::UTF8Decoder::ACCEPT: {
+                                auto ret = m_parser.feed(m_utf8_decoder.codepoint());
+                                if (G_UNLIKELY(ret < 0)) {
+                                        g_printerr("Parser error!\n");
+                                        return bufend;
+                                }
+
+                                m_seq_stats[ret]++;
+                                if (ret != VTE_SEQ_NONE) {
+                                        m_cmd_stats[seq.command()]++;
+                                        if (!process_seq(seq))
+                                                return sptr;
+                                }
+                                break;
+                        }
+                        default:
+                                break;
+                        }
+                }
+
+                if (eos &&
+                    m_utf8_decoder.flush()) {
+                        auto ret = m_parser.feed(m_utf8_decoder.codepoint());
+                        if (G_UNLIKELY(ret < 0)) {
+                                g_printerr("Parser error!\n");
+                                return bufend;
+                        }
+
+                        m_seq_stats[ret]++;
+                        if (ret != VTE_SEQ_NONE) {
+                                m_cmd_stats[seq.command()]++;
+                                if (!process_seq(seq))
+                                        return bufend;
+                        }
+                }
+
+                return bufend;
+        }
+
+        void
+        process_fd(int fd)
+        {
+                auto buf = g_new0(uint8_t, m_buffer_size);
 
                 auto start_time = g_get_monotonic_time();
 
-                vte::base::UTF8Decoder decoder;
-
-                gsize buf_start = 0;
+                std::memset(buf, 0, k_buf_overlap);
+                auto buf_start = k_buf_overlap;
                 for (;;) {
-                        auto len = read(fd, buf + buf_start, buf_size - buf_start);
-                        if (!len)
-                                break;
+                        auto len = read(fd, buf + buf_start, m_buffer_size - buf_start);
                         if (len == -1) {
                                 if (errno == EAGAIN)
                                         continue;
                                 break;
                         }
 
-                        auto const bufend = buf + len;
-                        for (auto sptr = buf; sptr < bufend; ++sptr) {
-                                switch (decoder.decode(*sptr)) {
-                                case vte::base::UTF8Decoder::REJECT_REWIND:
-                                        /* Rewind the stream.
-                                         * Note that this will never lead to a loop, since in the
-                                         * next round this byte *will* be consumed.
-                                         */
-                                        --sptr;
-                                        [[fallthrough]];
-                                case vte::base::UTF8Decoder::REJECT:
-                                        decoder.reset();
-                                        /* Fall through to insert the U+FFFD replacement character. */
-                                        [[fallthrough]];
-                                case vte::base::UTF8Decoder::ACCEPT: {
-                                        auto ret = parser.feed(decoder.codepoint());
-                                        if (G_UNLIKELY(ret < 0)) {
-                                                g_printerr("Parser error!\n");
-                                                goto out;
-                                        }
+                        auto const eos = (len == 0);
+                        uint8_t const* bufstart = buf + buf_start;
+                        auto const bufend = bufstart + len;
 
-                                        m_seq_stats[ret]++;
-                                        if (ret != VTE_SEQ_NONE) {
-                                                m_cmd_stats[seq.command()]++;
-                                                func(seq);
-                                        }
+                        for (auto sptr = bufstart; ; ) {
+                                switch (m_current_data_syntax) {
+                                case DataSyntax::ECMA48_UTF8:
+                                        sptr = process_data_utf8(sptr, bufend, eos);
                                         break;
-                                }
 
                                 default:
+                                        g_assert_not_reached();
                                         break;
                                 }
-                        }
-                }
 
-        out:
+                                if (sptr == bufend)
+                                        break;
+                        }
+
+                        if (eos)
+                                break;
+
+                        /* Chain buffers by copying data from end of buf to the start */
+                        std::memmove(buf, buf + buf_start + len - k_buf_overlap, k_buf_overlap);
+                }
 
                 int64_t time_spent = g_get_monotonic_time() - start_time;
                 g_array_append_val(m_bench_times, time_spent);
@@ -688,11 +839,9 @@ private:
                 g_free(buf);
         }
 
-        template<class Functor>
         bool
         process_file(int fd,
-                     int repeat,
-                     Functor& func)
+                     int repeat)
         {
                 if (fd == STDIN_FILENO && repeat != 1) {
                         g_printerr("Cannot consume STDIN more than once\n");
@@ -701,11 +850,13 @@ private:
 
                 for (auto i = 0; i < repeat; ++i) {
                         if (i > 0 && lseek(fd, 0, SEEK_SET) != 0) {
-                                g_printerr("Failed to seek: %m\n");
+                                auto errsv = vte::libc::ErrnoSaver{};
+                                g_printerr("Failed to seek: %s\n", g_strerror(errsv));
                                 return false;
                         }
 
-                        process_file_utf8(fd, func);
+                        reset();
+                        process_fd(fd);
                 }
 
                 return true;
@@ -713,7 +864,14 @@ private:
 
 public:
 
-        Processor() noexcept
+        Processor(Delegate& delegate,
+                  size_t buffer_size,
+                  bool statistics,
+                  bool benchmark) noexcept
+                : m_delegate{delegate},
+                  m_buffer_size{std::max(buffer_size, k_buf_overlap + 1)},
+                  m_statistics{statistics},
+                  m_benchmark{benchmark}
         {
                 memset(&m_seq_stats, 0, sizeof(m_seq_stats));
                 memset(&m_cmd_stats, 0, sizeof(m_cmd_stats));
@@ -722,14 +880,17 @@ public:
 
         ~Processor() noexcept
         {
+                if (m_statistics)
+                        print_statistics();
+                if (m_benchmark)
+                        print_benchmark();
+
                 g_array_free(m_bench_times, true);
         }
 
-        template<class Functor>
         bool
         process_files(char const* const* filenames,
-                      int repeat,
-                      Functor& func)
+                      int repeat)
         {
                 bool r = true;
                 if (filenames != nullptr) {
@@ -742,18 +903,21 @@ public:
                                 } else {
                                         fd = open(filename, O_RDONLY);
                                         if (fd == -1) {
-                                                g_printerr("Error opening file %s: %m\n", filename);
+                                                auto errsv = vte::libc::ErrnoSaver{};
+                                                g_printerr("Error opening file %s: %s\n",
+                                                           filename, g_strerror(errsv));
                                         }
                                 }
                                 if (fd != -1) {
-                                        r = process_file(fd, repeat, func);
-                                        close(fd);
+                                        r = process_file(fd, repeat);
+                                        if (fd != STDIN_FILENO)
+                                                close(fd);
                                         if (!r)
                                                 break;
                                 }
                         }
                 } else {
-                        r = process_file(STDIN_FILENO, repeat, func);
+                        r = process_file(STDIN_FILENO, repeat);
                 }
 
                 return r;
@@ -810,24 +974,9 @@ private:
         bool m_plain{false};
         bool m_quiet{false};
         bool m_statistics{false};
+        int m_buffer_size{16384};
         int m_repeat{1};
-        char** m_filenames{nullptr};
-
-        template<typename T1, typename T2 = T1>
-        class OptionArg {
-        private:
-                T1* m_return_ptr;
-                T2 m_value;
-        public:
-                OptionArg(T1* ptr, T2 v) : m_return_ptr{ptr}, m_value{v} { }
-                ~OptionArg() { *m_return_ptr = m_value; }
-
-                inline constexpr T2* ptr() noexcept { return &m_value; }
-        };
-
-        using BoolArg = OptionArg<bool, gboolean>;
-        using IntArg = OptionArg<int>;
-        using StrvArg = OptionArg<char**>;
+        vte::glib::StrvPtr m_filenames{};
 
 public:
 
@@ -835,62 +984,79 @@ public:
         Options(Options const&) = delete;
         Options(Options&&) = delete;
 
-        ~Options() {
-                if (m_filenames != nullptr)
-                        g_strfreev(m_filenames);
-        }
+        ~Options() = default;
 
-        inline constexpr bool benchmark()  const noexcept { return m_benchmark;  }
-        inline constexpr bool codepoints() const noexcept { return m_codepoints; }
-        inline constexpr bool lint()       const noexcept { return m_lint;       }
-        inline constexpr bool plain()      const noexcept { return m_plain;      }
-        inline constexpr bool quiet()      const noexcept { return m_quiet;      }
-        inline constexpr bool statistics() const noexcept { return m_statistics; }
-        inline constexpr int  repeat()     const noexcept { return m_repeat;     }
-        inline constexpr char const* const* filenames() const noexcept { return m_filenames; }
+        inline constexpr bool   benchmark()   const noexcept { return m_benchmark;  }
+        inline constexpr size_t buffer_size() const noexcept { return m_buffer_size; }
+        inline constexpr bool   codepoints()  const noexcept { return m_codepoints; }
+        inline constexpr bool   lint()        const noexcept { return m_lint;       }
+        inline constexpr bool   plain()       const noexcept { return m_plain;      }
+        inline constexpr bool   quiet()       const noexcept { return m_quiet;      }
+        inline constexpr bool   statistics()  const noexcept { return m_statistics; }
+        inline constexpr int    repeat()      const noexcept { return m_repeat;     }
+        inline char const* const* filenames() const noexcept { return m_filenames.get(); }
 
         bool parse(int argc,
                    char* argv[],
                    GError** error) noexcept
         {
-                BoolArg benchmark{&m_benchmark, false};
-                BoolArg codepoints{&m_codepoints, false};
-                BoolArg lint{&m_lint, false};
-                BoolArg plain{&m_plain, false};
-                BoolArg quiet{&m_quiet, false};
-                BoolArg statistics{&m_statistics, false};
-                IntArg repeat{&m_repeat, 1};
-                StrvArg filenames{&m_filenames, nullptr};
+                using BoolOption = vte::ValueGetter<bool, gboolean>;
+                using IntOption = vte::ValueGetter<int, int>;
+                using StrvOption = vte::ValueGetter<vte::glib::StrvPtr, char**, nullptr>;
+
+                auto benchmark = BoolOption{m_benchmark, false};
+                auto codepoints = BoolOption{m_codepoints, false};
+                auto lint = BoolOption{m_lint, false};
+                auto plain = BoolOption{m_plain, false};
+                auto quiet = BoolOption{m_quiet, false};
+                auto statistics = BoolOption{m_statistics, false};
+                auto buffer_size = IntOption{m_buffer_size, 16384};
+                auto repeat = IntOption{m_repeat, 1};
+                auto filenames = StrvOption{m_filenames, nullptr};
+
                 GOptionEntry const entries[] = {
-                        { "benchmark", 'b', 0, G_OPTION_ARG_NONE, benchmark.ptr(),
+                        { "benchmark", 'b', 0, G_OPTION_ARG_NONE, &benchmark,
                           "Measure time spent parsing each file", nullptr },
-                        { "codepoints", 'u', 0, G_OPTION_ARG_NONE, codepoints.ptr(),
+                        { "buffer-size", 'B', 0, G_OPTION_ARG_INT, &buffer_size,
+                          "Buffer size", "SIZE" },
+                        { "codepoints", 'u', 0, G_OPTION_ARG_NONE, &codepoints,
                           "Output unicode code points by number", nullptr },
-                        { "lint", 'l', 0, G_OPTION_ARG_NONE, lint.ptr(),
+                        { "lint", 'l', 0, G_OPTION_ARG_NONE, &lint,
                           "Check input", nullptr },
-                        { "plain", 'p', 0, G_OPTION_ARG_NONE, plain.ptr(),
+                        { "plain", 'p', 0, G_OPTION_ARG_NONE, &plain,
                           "Output plain text without attributes", nullptr },
-                        { "quiet", 'q', 0, G_OPTION_ARG_NONE, quiet.ptr(),
+                        { "quiet", 'q', 0, G_OPTION_ARG_NONE, &quiet,
                           "Suppress output except for statistics and benchmark", nullptr },
-                        { "repeat", 'r', 0, G_OPTION_ARG_INT, repeat.ptr(),
+                        { "repeat", 'r', 0, G_OPTION_ARG_INT, &repeat,
                           "Repeat each file COUNT times", "COUNT" },
-                        { "statistics", 's', 0, G_OPTION_ARG_NONE, statistics.ptr(),
+                        { "statistics", 's', 0, G_OPTION_ARG_NONE, &statistics,
                           "Output statistics", nullptr },
-                        { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, filenames.ptr(),
+                        { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &filenames,
                           nullptr, nullptr },
                         { nullptr },
                 };
 
-                auto context = g_option_context_new("[FILE…] — parser cat");
-                g_option_context_set_help_enabled(context, true);
-                g_option_context_add_main_entries(context, entries, nullptr);
+                auto context = vte::take_freeable(g_option_context_new("[FILE…] — parser cat"));
+                g_option_context_set_help_enabled(context.get(), true);
+                g_option_context_add_main_entries(context.get(), entries, nullptr);
 
-                bool rv = g_option_context_parse(context, &argc, &argv, error);
-                g_option_context_free(context);
-                return rv;
+                return g_option_context_parse(context.get(), &argc, &argv, error);
         }
 }; // class Options
 
+template<class D>
+static bool
+process(Options const& options,
+        D delegate)
+{
+        auto proc = Processor{delegate,
+                              options.buffer_size(),
+                              options.statistics(),
+                              options.benchmark()};
+
+        return proc.process_files(options.filenames(), options.repeat());
+}
+
 int
 main(int argc,
      char *argv[])
@@ -899,30 +1065,24 @@ main(int argc,
         _vte_debug_init();
 
         Options options{};
-        GError* err = nullptr;
-        if (!options.parse(argc, argv, &err)) {
-                g_printerr("Failed to parse arguments: %s\n", err->message);
-                g_error_free(err);
+        auto error = vte::glib::Error{};
+        if (!options.parse(argc, argv, error)) {
+                g_printerr("Failed to parse arguments: %s\n", error.message());
                 return EXIT_FAILURE;
         }
 
-        bool rv;
-        Processor proc{};
+        auto rv = false;
         if (options.lint()) {
-                Linter linter{};
-                rv = proc.process_files(options.filenames(), 1, linter);
+                if (options.repeat() != 1) {
+                        g_printerr("Cannot use repeat option for linter\n");
+                } else {
+                        rv = process(options, Linter{});
+                }
         } else if (options.quiet()) {
-                Sink sink{};
-                rv = proc.process_files(options.filenames(), options.repeat(), sink);
+                rv = process(options, Sink{});
         } else {
-                PrettyPrinter pp{options.plain(), options.codepoints()};
-                rv = proc.process_files(options.filenames(), options.repeat(), pp);
+                rv = process(options, PrettyPrinter{options.plain(), options.codepoints()});
         }
 
-        if (options.statistics())
-                proc.print_statistics();
-        if (options.benchmark())
-                proc.print_benchmark();
-
         return rv ? EXIT_SUCCESS : EXIT_FAILURE;
 }
